@@ -79,6 +79,34 @@ namespace Banshee.Base
             
             return result;
         }
+
+        public static string FindProgramInPath (string command)
+        {
+            foreach (string path in GetExecPaths ()) {
+                string full_path = Path.Combine (path, command);
+                try {
+                    FileInfo info = new FileInfo (full_path);
+                    // FIXME: System.IO is super lame, should check for 0755
+                    if (info.Exists) {
+                        return full_path;
+                    }
+                } catch {
+                }
+            }
+
+            return null;
+        }
+
+        private static string [] GetExecPaths ()
+        {
+            string path = Environment.GetEnvironmentVariable ("PATH");
+            if (String.IsNullOrEmpty (path)) {
+                return new string [] { "/bin", "/usr/bin", "/usr/local/bin" };
+            }
+
+            // this is super lame, should handle quoting/escaping
+            return path.Split (':');
+        }
         
         public static string MakePathRelative (string path, string to)
         {
@@ -86,12 +114,20 @@ namespace Banshee.Base
                 return null;
             }
             
-            if (path.Length < to.Length + 1) {
+            if (path == to) {
+                return String.Empty;
+            }
+            
+            if (to[to.Length - 1] != Path.DirectorySeparatorChar) {
+                to = to + Path.DirectorySeparatorChar;
+            }
+            
+            if (path.Length < to.Length) {
                 return null;
             }
             
             return path.StartsWith (to)
-                ? path.Substring (to.Length + 1)
+                ? path.Substring (to.Length)
                 : null;
         }
         
@@ -129,6 +165,10 @@ namespace Banshee.Base
         public static string DefaultLibraryPath {
             get { return XdgBaseDirectorySpec.GetUserDirectory ("XDG_MUSIC_DIR", "Music"); }
         }
+
+        public static string SystemTempDir {
+            get { return "/tmp/"; }
+        }
         
         public static string TempDir {
             get {
@@ -144,6 +184,7 @@ namespace Banshee.Base
         }
         
         private static string cached_library_location;
+        private static string cached_library_location_with_separator;
         public static string LibraryLocation {
              get {
                 string path = LibrarySchema.Location.Get (Paths.DefaultLibraryPath);
@@ -157,6 +198,7 @@ namespace Banshee.Base
              
              set {
                 cached_library_location = value;
+                cached_library_location_with_separator = null;
                 LibrarySchema.Location.Set (cached_library_location); 
             }
         }
@@ -165,6 +207,15 @@ namespace Banshee.Base
             get { return cached_library_location ?? LibraryLocation; }
         }
         
+        public static string CachedLibraryLocationWithSeparator {
+            get {
+                if (cached_library_location_with_separator == null) {
+                    cached_library_location_with_separator = CachedLibraryLocation + Path.DirectorySeparatorChar;
+                }
+                return cached_library_location_with_separator;
+            }
+        }
+        
         private static string installed_application_prefix = null;
         public static string InstalledApplicationPrefix {
             get {
