@@ -143,8 +143,10 @@ namespace Banshee.Gui
 
             switch (args.Current) {
                 case PlayerState.Contacting:
+                case PlayerState.Loading:
+                case PlayerState.Loaded:
                 case PlayerState.Playing:
-                    ShowPlayAction ();
+                    ShowStopAction ();
                     break;
                 case PlayerState.Paused:
                     ShowPlay ();
@@ -160,23 +162,22 @@ namespace Banshee.Gui
             if (track != null) {
                 this["SeekToAction"].Sensitive = !track.IsLive;
                 this["RestartSongAction"].Sensitive = !track.IsLive;
-                this["RestartSongAction"].Label = (track.MediaAttributes & TrackMediaAttributes.VideoStream) == 0
-                    ? Catalog.GetString ("_Restart Song")
-                    : Catalog.GetString ("_Restart Video");
 
+                this["RestartSongAction"].Label = String.Format (Catalog.GetString ("_Restart {0}"), track.MediaTypeName);
+                this["JumpToPlayingTrackAction"].Label = String.Format (Catalog.GetString ("_Jump to Playing {0}"), track.MediaTypeName);
                 this["JumpToPlayingTrackAction"].Sensitive = true;
-                this["JumpToPlayingTrackAction"].Label = (track.MediaAttributes & TrackMediaAttributes.VideoStream) == 0
-                    ? Catalog.GetString ("_Jump to Playing Song")
-                    : Catalog.GetString ("_Jump to Playing Video");
             } else {
                 this["JumpToPlayingTrackAction"].Sensitive = false;
                 this["RestartSongAction"].Sensitive = false;
                 this["SeekToAction"].Sensitive = false;
             }
+
+            // Disable all actions while NotReady
+            Sensitive = args.Current != PlayerState.NotReady;
         }
         
-        private void ShowPlayAction ()
-        { 
+        private void ShowStopAction ()
+        {
             if (ServiceManager.PlayerEngine.CanPause) {
                 ShowPause ();
             } else {
