@@ -29,6 +29,7 @@
 using System;
 using System.Data;
 using System.IO;
+using System.Text.RegularExpressions;
 using System.Collections.Generic;
 using System.Threading;
 using Mono.Unix;
@@ -47,12 +48,29 @@ using Banshee.Metadata;
 using Banshee.MediaEngine;
 using Banshee.Collection;
 using Banshee.ServiceStack;
+using Banshee.PlaybackController;
  
 namespace Banshee.Lastfm.Radio
 {
-    public class StationSource : Source, ITrackModelSource, IUnmapableSource, IDisposable
+    public class StationSource : Source, ITrackModelSource, IUnmapableSource, IDisposable, IBasicPlaybackController
     {
         private static string generic_name = Catalog.GetString ("Last.fm Station");
+
+        public static StationSource CreateFromUrl (LastfmSource lastfm, string url)
+        {
+            foreach (StationType type in StationType.Types) {
+                string regex = Regex.Escape (type.GetStationFor ("XXX")).Replace ("XXX", "([^\\/]+)");
+                Match match = Regex.Match (url, regex);
+                if (match.Groups.Count == 2 && match.Groups[0].Captures.Count > 0) {
+                    Log.DebugFormat ("Creating last.fm station from url {0}", url);
+                    string arg = match.Groups[1].Captures[0].Value;
+                    StationSource station = new StationSource (lastfm, arg, type.Name, arg);
+                    lastfm.AddChildSource (station);
+                    station.NotifyUser ();
+                }
+            }
+            return null;
+        }
         
         private MemoryTrackListModel track_model;
         
@@ -92,15 +110,12 @@ namespace Banshee.Lastfm.Radio
 
         private int dbid;
         
-        protected override string TypeUniqueId {
-            get { return Convert.ToString (dbid); }
-        }
-        
         // For StationSources that already exist in the db
         protected StationSource (LastfmSource lastfm, int dbId, string name, string type, string arg, int playCount) : base (generic_name, name, 150)
         {
             this.lastfm = lastfm;
             dbid = dbId;
+            TypeUniqueId = dbid.ToString ();
             Type = StationType.FindByName (type);
             Arg = arg;
             PlayCount = playCount;
@@ -161,6 +176,7 @@ namespace Banshee.Lastfm.Radio
             );
 
             dbid = ServiceManager.DbConnection.Execute (command);
+            TypeUniqueId = dbid.ToString ();
         }
 
         private void Update ()
@@ -270,6 +286,32 @@ namespace Banshee.Lastfm.Radio
                     current_track = i;
             }
         }
+        
+#region IBasicPlaybackController
+
+        bool IBasicPlaybackController.First ()
+        {
+            return ((IBasicPlaybackController)this).Next (false);
+        }
+        
+        private bool playback_requested;    
+        bool IBasicPlaybackController.Next (bool restart)
+        {
+            TrackInfo next = NextTrack;
+            if (next != null) {
+                ServiceManager.PlayerEngine.OpenPlay (next);
+            }  else {
+                playback_requested = true;
+            }
+            return true;
+        }
+        
+        bool IBasicPlaybackController.Previous (bool restart)
+        {
+            return true;
+        }
+        
+#endregion
 
         public TrackInfo NextTrack {
             get { return GetTrack (current_track + 1); }
@@ -325,10 +367,12 @@ namespace Banshee.Lastfm.Radio
                             track_model.Reload ();
                             OnUpdated ();
 
-                            /*if (playback_requested) {
-                                StartPlayback ();
+                            if (playback_requested) {
+                                if (this == ServiceManager.PlaybackController.Source ) {
+                                    ((IBasicPlaybackController)this).Next (false);
+                                }
                                 playback_requested = false;
-                            }*/
+                            }
                         });
                     }
                 } else {
@@ -449,6 +493,10 @@ namespace Banshee.Lastfm.Radio
         public bool ShowBrowser {
             get { return false; }
         }
+        
+        public bool Indexable {
+            get { return false; }
+        }
 
 #endregion
 
